<?php
  /**
 * This file is part of a project maintained at  NC State University.
 *
 * More information is included in the project README.md.
 *
 * (c) NC State University
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace NCSU\Auth\Adapter;

use Exception;
use NCSU\Auth\AuthResult;
use NCSU\Auth\Exception\RuntimeException;
use NCSU\Auth\Http\Request;

/**
 * Authentication adapter for Shibboleth 2.5
 * @see https://shib.ncsu.edu/
 *
 * @author Dustin Wheeler <mdwheele@ncsu.edu>
 */
final class ShibAuthAdapter implements AdapterInterface
{
    const IDP_TEMPLATE = 'https://shib.ncsu.edu/Shibboleth.sso/Login?target=%s';

    protected $request;
    protected $identity;

    public function __construct(Request $request)
    {
        $this->request = $request;
    }

    /**
     * Attempts to authenticate and returns user to client page.
     *
     * @throws RuntimeException
     * @return \NCSU\Auth\AuthResult
     */
    public function authenticate()
    {
        if ( ! isset($this->request->server['Shib-Session-ID'])) {
            header(sprintf("Location: %s", $this->resolveLoginUriFromTemplate()));
        }

        // Technically, this is not specific enough. This is being done as a non-BC change
        // to allow previous WRAP users to not have to re-identify users with correct scopes.
        $this->identity = $this->request->server['SHIB_UID'];

        return new AuthResult(AuthResult::SUCCESS, $this->identity);
    }

    /**
     * @return string
     */
    private function resolveLoginUriFromTemplate()
    {
        $loginUri = sprintf(ShibAuthAdapter::IDP_TEMPLATE, $this->request->getCurrentUri());
        return $loginUri;
    }
}