<?php
  /**
 * This file is part of a project maintained at  NC State University.
 *
 * More information is included in the project README.md.
 *
 * (c) NC State University
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace NCSU\Auth\Adapter;

use Exception;
use NCSU\Auth\AuthResult;
use NCSU\Auth\Exception\RuntimeException;
use NCSU\Auth\Http\Request;

/**
 * Authentication adapter for WRAP 1.6
 * @see https://webauth.ncsu.edu/wrap/
 *
 * @author Dustin Wheeler <mdwheele@ncsu.edu>
 */
final class WrapAuthAdapter implements AdapterInterface
{
    const IDP = 'https://webauth.ncsu.edu/wrap-bin/was16.cgi';

    protected $request;
    protected $identity;

    public function __construct(Request $request)
    {
        $this->request = $request;
    }

    /**
     * Attempts to authenticate.
     *
     * @throws RuntimeException
     * @return \NCSU\Auth\AuthResult
     */
    public function authenticate()
    {
        if (headers_sent()) {
            throw new RuntimeException("You must call this adapter before headers are sent.");
        }

        if ( ! isset($this->request->server['WRAP_USERID'])) {
            setrawcookie('WRAP_REFERER', $this->request->getCurrentUri(), 0, '/', '.ncsu.edu');
            header(sprintf("Location: %s", WrapAuthAdapter::IDP));
            exit();
        }

        $this->identity = $this->request->server['WRAP_USERID'];

        return new AuthResult(AuthResult::SUCCESS, $this->identity);
    }
}
